# This file was auto-generated by Fern from our API Definition.

import typing

from .image_content import ImageContent
from .text_content import TextContent

LettaUserMessageContentUnion = typing.Union[TextContent, ImageContent]
