# This file was auto-generated by Fern from our API Definition.

import typing

CompletionCreateParamsStreamingModel = typing.Union[
    str,
    typing.Literal["gpt-5"],
    typing.Literal["gpt-5-mini"],
    typing.Literal["gpt-5-nano"],
    typing.Literal["gpt-5-2025-08-07"],
    typing.Literal["gpt-5-mini-2025-08-07"],
    typing.Literal["gpt-5-nano-2025-08-07"],
    typing.Literal["gpt-5-chat-latest"],
    typing.Literal["gpt-4.1"],
    typing.Literal["gpt-4.1-mini"],
    typing.Literal["gpt-4.1-nano"],
    typing.Literal["gpt-4.1-2025-04-14"],
    typing.Literal["gpt-4.1-mini-2025-04-14"],
    typing.Literal["gpt-4.1-nano-2025-04-14"],
    typing.Literal["o4-mini"],
    typing.Literal["o4-mini-2025-04-16"],
    typing.Literal["o3"],
    typing.Literal["o3-2025-04-16"],
    typing.Literal["o3-mini"],
    typing.Literal["o3-mini-2025-01-31"],
    typing.Literal["o1"],
    typing.Literal["o1-2024-12-17"],
    typing.Literal["o1-preview"],
    typing.Literal["o1-preview-2024-09-12"],
    typing.Literal["o1-mini"],
    typing.Literal["o1-mini-2024-09-12"],
    typing.Literal["gpt-4o"],
    typing.Literal["gpt-4o-2024-11-20"],
    typing.Literal["gpt-4o-2024-08-06"],
    typing.Literal["gpt-4o-2024-05-13"],
    typing.Literal["gpt-4o-audio-preview"],
    typing.Literal["gpt-4o-audio-preview-2024-10-01"],
    typing.Literal["gpt-4o-audio-preview-2024-12-17"],
    typing.Literal["gpt-4o-audio-preview-2025-06-03"],
    typing.Literal["gpt-4o-mini-audio-preview"],
    typing.Literal["gpt-4o-mini-audio-preview-2024-12-17"],
    typing.Literal["gpt-4o-search-preview"],
    typing.Literal["gpt-4o-mini-search-preview"],
    typing.Literal["gpt-4o-search-preview-2025-03-11"],
    typing.Literal["gpt-4o-mini-search-preview-2025-03-11"],
    typing.Literal["chatgpt-4o-latest"],
    typing.Literal["codex-mini-latest"],
    typing.Literal["gpt-4o-mini"],
    typing.Literal["gpt-4o-mini-2024-07-18"],
    typing.Literal["gpt-4-turbo"],
    typing.Literal["gpt-4-turbo-2024-04-09"],
    typing.Literal["gpt-4-0125-preview"],
    typing.Literal["gpt-4-turbo-preview"],
    typing.Literal["gpt-4-1106-preview"],
    typing.Literal["gpt-4-vision-preview"],
    typing.Literal["gpt-4"],
    typing.Literal["gpt-4-0314"],
    typing.Literal["gpt-4-0613"],
    typing.Literal["gpt-4-32k"],
    typing.Literal["gpt-4-32k-0314"],
    typing.Literal["gpt-4-32k-0613"],
    typing.Literal["gpt-3.5-turbo"],
    typing.Literal["gpt-3.5-turbo-16k"],
    typing.Literal["gpt-3.5-turbo-0301"],
    typing.Literal["gpt-3.5-turbo-0613"],
    typing.Literal["gpt-3.5-turbo-1106"],
    typing.Literal["gpt-3.5-turbo-0125"],
    typing.Literal["gpt-3.5-turbo-16k-0613"],
]
