# This file was auto-generated by Fern from our API Definition.

import typing

from .dynamic_manager import DynamicManager
from .round_robin_manager import RoundRobinManager
from .sleeptime_manager import SleeptimeManager
from .supervisor_manager import SupervisorManager
from .voice_sleeptime_manager import VoiceSleeptimeManager

GroupSchemaManagerConfig = typing.Union[
    DynamicManager, RoundRobinManager, SleeptimeManager, SupervisorManager, VoiceSleeptimeManager
]
