# This file was auto-generated by Fern from our API Definition.

import typing

from .json_object_response_format import JsonObjectResponseFormat
from .json_schema_response_format import JsonSchemaResponseFormat
from .text_response_format import TextResponseFormat

LettaSchemasAgentFileAgentSchemaResponseFormat = typing.Union[
    JsonObjectResponseFormat, JsonSchemaResponseFormat, TextResponseFormat
]
