# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel


class TemplatesGetTemplateSnapshotResponseBlocksItem(UncheckedBaseModel):
    label: str
    value: str
    limit: float
    description: str
    preserve_on_migration: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="preserveOnMigration")
    ] = None
    read_only: typing_extensions.Annotated[bool, FieldMetadata(alias="readOnly")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
