# This file was auto-generated by Fern from our API Definition.

import typing

ConnectMcpServerResponseEvent = typing.Union[
    typing.Literal["connection_attempt", "success", "error", "oauth_required", "authorization_url", "waiting_for_auth"],
    typing.Any,
]
