# This file was auto-generated by Fern from our API Definition.

import typing

from .custom_format_grammar import CustomFormatGrammar
from .custom_format_text import CustomFormatText

OpenaiTypesChatChatCompletionCustomToolParamCustomFormat = typing.Union[CustomFormatText, CustomFormatGrammar]
