# This file was auto-generated by Fern from our API Definition.

import typing

from .e_2_b_sandbox_config import E2BSandboxConfig
from .local_sandbox_config import LocalSandboxConfig
from .modal_sandbox_config import ModalSandboxConfig

SandboxConfigCreateConfig = typing.Union[LocalSandboxConfig, E2BSandboxConfig, ModalSandboxConfig]
