# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_custom_tool_param import ChatCompletionCustomToolParam
from .chat_completion_function_tool_param import ChatCompletionFunctionToolParam

CompletionCreateParamsStreamingToolsItem = typing.Union[ChatCompletionFunctionToolParam, ChatCompletionCustomToolParam]
