# This file was auto-generated by Fern from our API Definition.

import typing

from .templates_get_template_snapshot_response_agents_item_tool_rules_item_child_output_mapping import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_five import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_four import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_max_count_limit import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_one import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_seven import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_two import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo,
)
from .templates_get_template_snapshot_response_agents_item_tool_rules_item_zero import (
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero,
)

TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItem = typing.Union[
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven,
]
