# This file was auto-generated by Fern from our API Definition.

import typing

CompletionCreateParamsStreamingServiceTier = typing.Union[
    typing.Literal["auto", "default", "flex", "scale", "priority"], typing.Any
]
