# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel
from .templates_get_template_snapshot_response_agents_item_memory_variables_data_item import (
    TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem,
)


class TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariables(UncheckedBaseModel):
    version: str
    data: typing.List[TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
