# This file was auto-generated by Fern from our API Definition.

import typing

TemplatesGetTemplateSnapshotResponseType = typing.Union[
    typing.Literal["classic", "cluster", "sleeptime", "round_robin", "supervisor", "dynamic", "voice_sleeptime"],
    typing.Any,
]
