# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_content_part_text_param import ChatCompletionContentPartTextParam

ChatCompletionDeveloperMessageParamContent = typing.Union[str, typing.List[ChatCompletionContentPartTextParam]]
