# This file was auto-generated by Fern from our API Definition.

import typing

ProviderType = typing.Union[
    typing.Literal[
        "anthropic",
        "azure",
        "bedrock",
        "cerebras",
        "deepseek",
        "google_ai",
        "google_vertex",
        "groq",
        "hugging-face",
        "letta",
        "lmstudio_openai",
        "mistral",
        "ollama",
        "openai",
        "together",
        "vllm",
        "xai",
    ],
    typing.Any,
]
