# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_message_custom_tool_call_param import ChatCompletionMessageCustomToolCallParam
from .chat_completion_message_function_tool_call_param import ChatCompletionMessageFunctionToolCallParam

ChatCompletionAssistantMessageParamToolCallsItem = typing.Union[
    ChatCompletionMessageFunctionToolCallParam, ChatCompletionMessageCustomToolCallParam
]
